import json, logging, std/httpclient, sequtils, strutils

import ./models, ./service

proc makeHttpClient(cfg: HffEntryFormsApiConfig): HttpClient =
  let headers = newHttpHeaders([
    ("Content-Type", "application/json"),
    ("Authorization", "Bearer " & cfg.integrationToken),
    ("Notion-Version", cfg.notionVersion)
  ], true)
  debug $headers
  return newHttpClient(headers = headers, )

proc getEventProposalConfig*(cfg: HffEntryFormsApiConfig): EventProposalConfig =
  let http = makeHttpClient(cfg)

  let apiResp = http.get(cfg.notionApiBaseUrl & "/databases/" & cfg.eventParentId)
  debug apiResp.status

  if not apiResp.status.startsWith("2"):
    debug apiResp.body
    raiseApiError(Http500,
      "unable to read event propsal configuration from notion API")

  let bodyJson = parseJson(apiResp.body)
  let departmentOptionsJson = bodyJson{
    "properties", "Department", "multi_select", "options"}

  if departmentOptionsJson.isNil:
    raiseApiError(Http500,
      "missing read department values from Notion API-supplied event schema")

  return EventProposalConfig(
    departmentOptions: departmentOptionsJson.toSeq.mapIt(
      ( value: it["name"].getStr, color: it["color"].getStr )
    )
  )

proc createProposedEvent*(cfg: HffEntryFormsApiConfig, ep: EventProposal): bool =
  let http = makeHttpClient(cfg)
  let epNotionPage = ep.asNotionPage
  epNotionPage["parent"] = %*{ "database_id": cfg.eventParentId }

  let apiResp = http.post(cfg.notionApiBaseUrl & "/pages", $epNotionPage)

  debug apiResp.status
  if not apiResp.status.startsWith("2"): debug apiResp.body

  return apiResp.status.startsWith("2")
